import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'database.dart';

// ignore: must_be_immutable
class CustomScrollController<T extends AbstractDataModel<T>>
    extends StatefulWidget {
  final Widget Function(T, int) builder;
  final Database<T> db;
  int page;
  int total;
  int limit;

  CustomScrollController({
    @required this.db,
    @required this.builder,
    this.page: 1,
    this.total: 0,
    this.limit: 100,
  });

  @override
  State<StatefulWidget> createState() {
    return _CustomScrollControllerState<T>();
  }
}

class _CustomScrollControllerState<T extends AbstractDataModel<T>>
    extends State<CustomScrollController> {
  BuildContext context;
  ScrollController scrollController;
  List<T> list = List<T>.empty(growable: true);

  @override
  void initState() {
    this.scrollController = ScrollController();
    this.scrollController.addListener(scrollListener);
    this.getItemList();
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    return Container(
      child: Expanded(
        child: ListView.builder(
          controller: this.scrollController,
          itemCount: list.length,
          itemBuilder: (BuildContext context, int index) {
            return widget.builder(list[index], index);
          },
        ),
      ),
    );
  }

  void getItemList() async {
    widget.total = await widget.db.count();

    List<T> _list = await widget.db.paginate(
      page: widget.page,
      limit: widget.limit,
    );
    print("Total entries: ${widget.total}");

    if (_list.length > 0) {
      this.list.addAll(_list);
      setState(() {
        widget.page += 1;
      });
    }
  }

  void scrollListener() {
    if (scrollController.offset >= scrollController.position.maxScrollExtent &&
        !scrollController.position.outOfRange) {
      setState(
        () {
          print("Yekonga: => Reached bottom");
          getItemList();
        },
      );
    }
    if (scrollController.offset <= scrollController.position.minScrollExtent &&
        !scrollController.position.outOfRange) {
      setState(
        () {
          print("Yekonga: => Reached top");
        },
      );
    }
  }
}
